#!/usr/bin/env python3

# Copyright 2017 The Imaging Source Europe GmbH
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
# http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

#
# This example will show you how to list information about the available devices
#

import sys
import gi

gi.require_version("Tcam", "0.1")
gi.require_version("Gst", "1.0")

from gi.repository import Tcam, Gst


def list_devices():
    """
    Print information about all  available devices
    """
    source = Gst.ElementFactory.make("tcambin")

    serials = source.get_device_serials_backend()

    for single_serial in serials:

        # This returns someting like:
        # (True,
        #  name='DFK Z12GP031',
        #  identifier='The Imaging Source Europe GmbH-11410533',
        #  connection_type='aravis')
        # The identifier is the name given by the backend
        # The connection_type identifies the backend that is used.
        #     Currently 'aravis', 'v4l2', 'libusb' and 'unknown' exist
        (return_value, model,
         identifier, connection_type) = source.get_device_info(single_serial)

        # return value would be False when a non-existant serial is used
        # since we are iterating get_device_serials this should not happen
        if return_value:

            print("Model: {} Serial: {} Type: {}".format(model,
                                                         single_serial,
                                                         connection_type))


if __name__ == "__main__":

    Gst.init(sys.argv)  # init gstreamer
    list_devices()
